from __future__ import unicode_literals
from django.core.files.uploadedfile import InMemoryUploadedFile
from django.utils.encoding import python_2_unicode_compatible
from django.contrib.staticfiles.storage import staticfiles_storage


@python_2_unicode_compatible
class BaseTag(object):
    """
    Base Tag behaviour
    """
    tag_name = ""
    self_closed = True  # Defines if should close itself < />
    meta_name = ""
    meta_content = ""
    tag_value = ""

    def __init__(self, tag_name=None, self_closed=None, meta_name=None, meta_content=None, tag_value=None):
        """
        Explicit Initialization of Tag
        :param tag_name:
        :param self_closed:
        :param meta_name:
        :param meta_content:
        :param tag_value:
        :return:
        """
        if tag_name:
            self.tag_name = tag_name
        if tag_value:
            self.tag_value = tag_value
        if self_closed is not None:
            self.self_closed = self_closed
        if meta_name:
            self.meta_name = meta_name
        if meta_content:
            self.meta_content = meta_content

    def print_tag(self):
        """
        Builds tag as text for printing
        :return: text
        """
        if not isinstance(self, BaseTag) and not issubclass(self, BaseTag):
            raise TypeError("Tag must be of class simple-seo.tags.BaseTag")

        if self.self_closed:
            if self.meta_name and self.meta_content:
                return "<%s name=\"%s\" content=\"%s\" />" % (self.tag_name, self.meta_name, self.meta_content)
            else:
                return ""
        else:
            if self.tag_value:
                return "<%s>%s</%s>" % (self.tag_name, self.tag_value, self.tag_name)
            else:
                return ""

    def __str__(self):
        if self.self_closed:
            if self.meta_content is None:
                return ''
            return self.meta_content
        else:
            if self.tag_value is None:
                return ''
            return self.tag_value

    def __len__(self):
        raise NotImplementedError("Must implement tag output __len__()")


class TitleTag(BaseTag):
    """
    Title Tag class
    """

    def __init__(self, tag_name=None, self_closed=None, meta_name=None, meta_content=None, tag_value=None, *args,
                 **kwargs):
        super(TitleTag, self).__init__(tag_name=tag_name, self_closed=self_closed, meta_name=meta_name,
                                       meta_content=meta_content, tag_value=tag_value)
        self.tag_name = "title"
        self.self_closed = False
        if 'value' in kwargs:
            if len(kwargs['value']) > 68:
                self.tag_value = kwargs['value'][:68]
            else:
                self.tag_value = kwargs['value']

    def __str__(self):
        if self.tag_value is None:
            return ''
        return self.tag_value

    def __len__(self):
        if self.tag_value is None:
            return 0
        return len(self.tag_value)


class BaseMetatag(BaseTag):
    """
    Base Meta Tag
    """

    def __init__(self, tag_name=None, self_closed=None, meta_name=None, meta_content=None, tag_value=None, *args,
                 **kwargs):
        super(BaseMetatag, self).__init__(tag_name=tag_name, self_closed=self_closed, meta_name=meta_name,
                                          meta_content=meta_content, tag_value=tag_value)

        self.tag_name = 'meta'
        self.self_closed = True
        if 'name' in kwargs:
            self.meta_name = kwargs['name']

    def __str__(self):
        if self.meta_content is None:
            return ''
        return self.meta_content

    def __len__(self):
        if self.meta_content is None:
            return 0
        return len(self.meta_content)


class MetaTag(BaseMetatag):
    """
    Meta Tag class
    """

    def __init__(self, tag_name=None, self_closed=None, meta_name=None, meta_content=None, tag_value=None, *args,
                 **kwargs):
        super(BaseMetatag, self).__init__(tag_name=tag_name, self_closed=self_closed, meta_name=meta_name,
                                          meta_content=meta_content, tag_value=tag_value)
        self.tag_name = 'meta'
        if 'value' in kwargs:
            if kwargs['value'] and len(kwargs['value']) > 255:
                self.meta_content = kwargs['value'][:255]
            else:
                self.meta_content = kwargs['value']


class ImageMetaTag(BaseMetatag):
    """
    Image Meta Tag class
    """

    def __init__(self, tag_name=None, self_closed=None, meta_name=None, meta_content=None, tag_value=None, *args,
                 **kwargs):
        super(ImageMetaTag, self).__init__(tag_name=tag_name, self_closed=self_closed, meta_name=meta_name,
                                           meta_content=meta_content, tag_value=tag_value)
        self.tag_name = 'meta'
        if 'value' in kwargs:
            if kwargs['value']:
                if isinstance(kwargs['value'], InMemoryUploadedFile):
                    rel_path = kwargs['path'] + kwargs['value'].name
                    self._inmemoryuploadedfile = kwargs['value']
                else:
                    self._inmemoryuploadedfile = None
                    rel_path = kwargs['value']
                if staticfiles_storage:
                    self.meta_content = staticfiles_storage.url(rel_path)
                else:
                    self.meta_content = rel_path

    @property
    def url(self):
        return self.meta_content

    def __str__(self):
        if self.meta_content is None:
            return ''
        return self.meta_content


class KeywordsTag(BaseMetatag):
    """
    Keywords Meta Tag class
    """

    def _clean(self, value):
        if value:
            return value.replace('"', '&#34;').replace("\n", ", ").strip()
        else:
            return value

    def __init__(self, tag_name=None, self_closed=None, meta_name=None, meta_content=None, tag_value=None, *args,
                 **kwargs):
        super(KeywordsTag, self).__init__(tag_name=tag_name, self_closed=self_closed, meta_name=meta_name,
                                          meta_content=meta_content, tag_value=tag_value)
        self.tag_name = 'meta'
        if 'value' in kwargs:
            if kwargs['value'] and len(kwargs['value']) > 255:
                self.meta_content = self._clean(kwargs['value'][:255])
            else:
                self.meta_content = self._clean(kwargs['value'])



from django.test import TestCase
from selenium.webdriver.firefox.webdriver import WebDriver as FirefoxDriver
from django.test.testcases import LiveServerTestCase

from testapp.models import MyMetadata
from simple_seo.tags import TitleTag, MetaTag, KeywordsTag, BaseTag


class TagPrintingTest(TestCase):
    """
    Testing Tag objects printing functions
    """
    base_tag_selfclosed = None
    base_tag_notselfclosed = None

    def setUp(self):
        self.base_tag_notselfclosed = BaseTag(tag_name='test', self_closed=False, tag_value='Test Tag',
                                              meta_name='meta name',
                                              meta_content='meta content')
        self.base_tag_selfclosed = BaseTag(tag_name='test', self_closed=True, tag_value='Test Tag',
                                           meta_name='meta name',
                                           meta_content='meta content')

    def tearDown(self):
        self.base_tag_selfclosed = None
        self.base_tag_notselfclosed = None

    def test_notselfclosed_tag(self):
        self.assertEqual(self.base_tag_notselfclosed.print_tag(), '<test>Test Tag</test>')

    def test_selfclosed_tag(self):
        self.assertEqual(self.base_tag_selfclosed.print_tag(), '<test name="meta name" content="meta content" />')


class FieldPrintingTest(LiveServerTestCase):
    """
    Testing TagFields objects printing functions
    """

    def setUp(self):
        self.mymetadata = MyMetadata()
        self.mymetadata.view_name = 'template_test'
        self.mymetadata.title = TitleTag(tag_value='Title Test')
        self.mymetadata.author = MetaTag(meta_name='author', meta_content='Test Author')
        self.mymetadata.description = MetaTag(meta_name='description', meta_content='Test Description')
        self.mymetadata.keywords = KeywordsTag(meta_name='keywords', meta_content='test, author')
        self.mymetadata.save()

        self.firefox = FirefoxDriver()

    def tearDown(self):
        self.firefox.quit()

    def test_title_rendering(self):
        self.firefox.get('%s%s' % (self.live_server_url, '/test/'))
        title_element = self.firefox.title
        self.assertIsNotNone(title_element)
        self.assertEqual(title_element, 'Title Test')

    def test_metatags_rendering(self):
        self.firefox.get('%s%s' % (self.live_server_url, '/test/'))
        author_element = self.firefox.find_element_by_xpath('/html/head/meta[@name=\'author\']')
        self.assertIsNotNone(author_element)
        self.assertEqual(author_element.get_attribute('content'), 'Test Author')
        description_element = self.firefox.find_element_by_xpath('/html/head/meta[@name=\'description\']')
        self.assertIsNotNone(description_element)
        self.assertEqual(description_element.get_attribute('content'), 'Test Description')
        keywords_element = self.firefox.find_element_by_xpath('/html/head/meta[@name=\'keywords\']')
        self.assertIsNotNone(keywords_element)
        self.assertEqual(keywords_element.get_attribute('content'), 'test, author')



from __future__ import print_function
from django.core.exceptions import ImproperlyConfigured
from django.core.urlresolvers import RegexURLResolver, RegexURLPattern
import logging
from django.conf import settings

log = logging.getLogger(__name__)


def _load_pattern(views, pattern, namespace=None):
    """
    Computes RegexURLPattern
    :param pattern: RegexURLPattern
    :param namespace: str
    :return: tuple
    """
    name = getattr(pattern, 'name', None)
    if name:
        if namespace:
            views.append((namespace + ':' + name, namespace + ':' + name))
        else:
            views.append((name, name))


def _load_patterns(views, patterns, namespace=None):
    """
    Computes RegexURLResolver
    :param views: list
    :param patterns: RegexURLResolver
    :param namespace: str
    :return:
    """
    for pattern in patterns:
        if isinstance(pattern, RegexURLPattern):
            _load_pattern(views, pattern, namespace)
        elif isinstance(pattern, RegexURLResolver):
            if namespace and hasattr(pattern, 'namespace'):
                namespace += ':' + pattern.namespace
            elif hasattr(pattern, 'namespace'):
                namespace = pattern.namespace
            _load_patterns(views, pattern.url_patterns, namespace)
        else:
            pass


def load_view_names(urlconf=None):
    """
    Loads view names
    Warning: Only named views are loaded
    :return: generator
    """
    if not urlconf:
        try:
            urlconf = __import__(settings.ROOT_URLCONF, {}, {}, [''])
        except Exception as e:
            raise ImproperlyConfigured("Error occurred while trying to load %s: %s" % (settings.ROOT_URLCONF, str(e)))
    views = []
    for p in urlconf.urlpatterns:
        if isinstance(p, RegexURLPattern):
            _load_pattern(views, p)
        elif isinstance(p, RegexURLResolver):
            _load_patterns(views, p.url_patterns, getattr(p, 'namespace', None))
        else:
            pass

    return views




